@@ -8,7 +8,6 @@ use colored::*;
 use git2::ErrorCode;
 use lazy_static::lazy_static;
 use log::{debug, error, info, trace};
-use logchop::*;
 use std::io::{self, stdin, stdout, Cursor, Write};
 use std::{env, include_str, process};
 use tabwriter::TabWriter;
@@ -23,6 +22,11 @@ lazy_static! {
     };
 }
 
+fn abort(message: &str) -> ! {
+    eprintln!("{}", message.red());
+    process::exit(1);
+}
+
 /// Get the remote for the current project
 fn get_remote(remote_name: &str, fetch_api_key: bool) -> Result<Box<dyn remotes::Remote>> {
     let remote_url = git::get_remote_url(remote_name);
@@ -32,15 +36,11 @@ fn get_remote(remote_name: &str, fetch_api_key: bool) -> Result<Box<dyn remotes:
 /// Get the remote, fail hard otherwise
 fn get_remote_hard(remote_name: &str, fetch_api_key: bool) -> Box<dyn remotes::Remote> {
     get_remote(remote_name, fetch_api_key).unwrap_or_else(|error| {
-        eprintln!(
-            "{}",
-            format!(
-                "There was a problem finding the remote Git repo: {}",
-                &error
-            )
-            .red()
+        let message = format!(
+            "There was a problem finding the remote Git repo: {}",
+            &error
         );
-        process::exit(1);
+        abort(&message);
     })
 }
 
@@ -50,31 +50,34 @@ fn checkout_mr(remote_name: &str, mr_id: i64) {
     let mut remote = get_remote_hard(remote_name, true);
     debug!("Found remote: {}", remote);
     let remote_branch_name = remote.get_remote_req_branch(mr_id).unwrap_or_else(|error| {
-        eprintln!(
-            "{}",
-            format!(
-                "There was a problem ascertaining the branch name: {}",
-                &error
-            )
-            .red()
+        let message = format!(
+            "There was a problem ascertaining the branch name: {}",
+            &error
         );
-        process::exit(1);
+        abort(&message);
     });
     debug!("Got remote branch name: {}", remote_branch_name);
-    git::checkout_branch(
+    match git::checkout_branch(
         remote_name,
         &remote_branch_name,
         &remote.get_local_req_branch(mr_id).unwrap(),
         remote.has_virtual_remote_branch_names(),
     )
-    .info_ok("Done")
     .unwrap_or_else(|err| {
-        eprintln!(
-            "{}",
-            format!("There was an error checking out the branch: {}", err).red()
-        );
-        process::exit(1)
-    });
+        let message = format!("There was an error checking out the branch: {}", err);
+        abort(&message);
+    }) {
+        git::CheckoutResult::BranchChanged => {
+            if git::push_current_ref(mr_id).is_err() {
+                trace!("Couldn't update the current ref");
+                eprintln!("{}", "failed to update some git-req metadata".yellow());
+            }
+        }
+        _ => {
+            eprintln!("Already on branch");
+        }
+    };
+    trace!("Done");
 }
 
 /// Clear the API key for the current domain
@@ -92,15 +95,11 @@ fn clear_domain_key(remote_name: &str) {
         Ok(_) => eprintln!("{}", "Domain key deleted!".green()),
         Err(e) => {
             error!("Git Config error: {}", e);
-            eprintln!(
-                "{}",
-                format!(
-                    "There was an error deleting the domain key: {}",
-                    e.message()
-                )
-                .red()
+            let message = format!(
+                "There was an error deleting the domain key: {}",
+                e.message()
             );
-            process::exit(1)
+            abort(&message);
         }
     }
 }
@@ -236,9 +235,18 @@ fn main() {
         app = build_cli();
         generate_completion(&mut app, &shell_name);
     } else {
-        checkout_mr(
-            &get_remote_name(&matches),
-            matches.value_of("REQUEST_ID").unwrap().parse().unwrap(),
-        );
+        let request_id = matches.value_of("REQUEST_ID").unwrap();
+        let mr_id = if request_id == "-" {
+            trace!("Received request for previous MR");
+            git::get_previous_mr_id().unwrap_or_else(|_| {
+                abort("Could not find previous request");
+            })
+        } else {
+            trace!("Received request for numbered MR: {}", request_id);
+            request_id.parse::<i64>().unwrap_or_else(|_| {
+                abort("Invalid request ID provided");
+            })
+        };
+        checkout_mr(&get_remote_name(&matches), mr_id);
     }
 }
